@@ -81,7 +81,7 @@ def func(dataset_dir, info_py="info.py", data_path="data", preds_path="predictio
         (output_dir).mkdir(parents=True, exist_ok=False)
 
         tmpl_mapping = {
-            "date": time.strftime("%Y-%m-%d %H:%M"),
+            "date": time.strftime(r"%Y-%m-%d %H:%M"),
             "aggregate_metrics": format_kv("mAP", mAP),
             "report": json.dumps(results.report(), indent=4),
         }
